@@ -310,7 +310,7 @@ check_nonce_nc (struct MHD_Connection *connection,
   np = nonce;
   while (*np != '\0')
     {
-      off = (off << 8) | (*np & (off >> 24));
+      off = (off << 8) | (*np ^ (off >> 24));
       np++;
     }
   off = off % mod;
@@ -690,6 +690,11 @@ MHD_basic_auth_get_username_password(struct MHD_Connection *connection,
       return NULL;
     }
   user = strdup(decode);
+  if (NULL == user)
+    {
+      free (decode);
+      return NULL;
+    }
   user[separator - decode] = '\0'; /* cut off at ':' */
   if (password != NULL) 
     {
